import { ServiceConfig as ServiceConfig$1 } from '@cleavelandprice/ngx-lib';
import * as i0 from '@angular/core';
import { inject, Injectable, Pipe, Component, Input, NgModule, Optional, SkipSelf } from '@angular/core';
import { of, map } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { DateTime } from 'luxon';
import { ActiveDirectoryService } from '@cleavelandprice/ngx-lib/active-directory';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

class ServiceConfig extends ServiceConfig$1 {
    // See the comment in the library's main service-config.ts file for an explanation
    constructor(useSsl_or_customApiUrl = false) {
        super(useSsl_or_customApiUrl);
        this.usersEndpoint = '/sharepoint/users';
        this.listItemsEndpoint = '/sharepoint/listitems';
        this.photos = {
            site: 'https://sharepoint.cleavelandprice.com/digitaldisplay',
            list: 'Employee Thumbnail Photos',
            additionalPropertiesToRetrieve: [
                'EncodedAbsUrl'
            ]
        };
        this.caching = {
            enabled: true,
            lifeMinutes: 720
        };
        this.logging = {
            dataCached: true,
            cachedDataReturned: true,
            freshDataReturned: true,
            cacheCleared: true
        };
    }
}

class SharePointService {
    //#region Utilities
    cacheExpired(cache) {
        if (!cache) {
            return true;
        }
        const lifeMinutes = this.config?.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
        const expiration = cache.lastUpdate.plus({ minutes: lifeMinutes });
        return DateTime.now() > expiration;
    }
    //#endregion
    //#region Initialization
    constructor() {
        //#region Services
        this.config = inject(ServiceConfig, { optional: true }) ?? new ServiceConfig();
        this.http = inject(HttpClient);
        this.defaultCacheLifeMinutes = 720;
        this.configureCaching();
        this.configureLogging();
    }
    configureCaching() {
        // If caching config is missing entirely, give it a default
        if (this.config) {
            this.config.caching = this.config?.caching ?? {
                enabled: false,
                lifeMinutes: this.defaultCacheLifeMinutes
            };
        }
        this.cache = {
            photos: undefined,
            users: undefined
        };
    }
    configureLogging() {
        // If logging config is missing entirely, give it a default
        if (this.config) {
            this.config.logging = this.config.logging ?? {
                dataCached: false,
                cachedDataReturned: false,
                freshDataReturned: false,
                cacheCleared: false
            };
        }
    }
    //#endregion
    getListItems(request) {
        const apiUrl = this.config?.apiUrl;
        const endpoint = this.config?.listItemsEndpoint;
        return this.http.post(`${apiUrl}${endpoint}`, request);
    }
    getPhotos() {
        const request = {
            username: this.config?.username,
            password: this.config?.password,
            site: this.config?.photos.site,
            list: this.config?.photos.list,
            additionalPropertiesToRetrieve: this.config?.photos.additionalPropertiesToRetrieve
        };
        const observable$ = this.getListItems(request);
        if (this.config?.caching?.enabled) {
            if (this.cacheExpired(this.cache?.photos)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.photos = {
                        data,
                        lastUpdate: DateTime.now()
                    };
                    if (this.config?.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Photos cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached photos');
                }
                return of(this.cache.photos.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config?.logging?.freshDataReturned) {
            console.log('Returning fresh photos');
        }
        return observable$;
    }
    getUsers(request, id) {
        const apiUrl = this.config?.apiUrl;
        const endpoint = this.config?.usersEndpoint;
        const observable$ = this.http.post(`${apiUrl}${endpoint}/${id ?? ''}`, request);
        //#region Single user
        if (id) {
            return observable$;
        }
        //#endregion
        //#region Multiple users
        if (this.config?.caching?.enabled) {
            if (this.cacheExpired(this.cache.users)) {
                // Get the data and cache it
                observable$.subscribe(data => {
                    this.cache.users = {
                        data: data,
                        lastUpdate: DateTime.now()
                    };
                    if (this.config?.logging?.dataCached) {
                        const lifeMinutes = this.config.caching?.lifeMinutes ?? this.defaultCacheLifeMinutes;
                        console.log(`Users cached for ${lifeMinutes} minutes`);
                    }
                });
            }
            else {
                // Cache exists and hasn't expired yet
                if (this.config.logging?.cachedDataReturned) {
                    console.log('Returning cached users');
                }
                return of(this.cache.users.data);
            }
        }
        // Cache is either disabled or expired, return data from API
        if (this.config?.logging?.freshDataReturned) {
            console.log('Returning fresh users');
        }
        return observable$;
        //#endregion
    }
    clearCache() {
        this.cache = undefined;
        if (this.config?.logging?.cacheCleared) {
            console.log('Cache cleared');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

class EmployeePhotoUrlPipe {
    constructor() {
        this.sharePointService = inject(SharePointService);
    }
    transform(user, userPhotos) {
        const observable$ = userPhotos ? of(userPhotos) : this.sharePointService.getPhotos();
        return observable$
            .pipe(map(photos => photos.find(pic => 
        // Try to match by employeeNumber, accountName, or title
        (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()))?.encodedAbsUrl));
        /*const photo = photos
          .find(pic =>
            (pic.employeeNumber && user.employeeNumber && +pic.employeeNumber === +user.employeeNumber) ||
            (pic.accountName && pic.accountName.toLowerCase() === user.sAMAccountName.toLowerCase()) ||
            (pic.title && pic.title.toLowerCase() === user.displayName.toLowerCase()));
    
        if (!photo) {
          console.log(`No photo match for ${user.displayName}`);
        }
    
        return photo ? photo.encodedAbsUrl : null;*/
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: EmployeePhotoUrlPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe }); }
    static { this.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "17.1.2", ngImport: i0, type: EmployeePhotoUrlPipe, name: "employeePhotoUrl" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: EmployeePhotoUrlPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'employeePhotoUrl'
                }]
        }] });

class EmployeePhotoComponent {
    constructor() {
        this.activeDirectoryService = inject(ActiveDirectoryService);
        this.size = "2em";
        this.notRound = false;
    }
    ngOnInit() {
        if (this.accountName && !this.employee) {
            // Developer passed in an account name instead of a user object
            // We need to do the work of getting the user object ourselves
            this.activeDirectoryService.getUser(this.accountName)
                .subscribe(user => this.employee = user);
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: EmployeePhotoComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.1.2", type: EmployeePhotoComponent, selector: "employee-photo", inputs: { employee: "employee", accountName: "accountName", employeePhotos: "employeePhotos", size: "size", notRound: "notRound" }, ngImport: i0, template: "@if (employee) {\n  @if ((employee | employeePhotoUrl:employeePhotos | async)) {\n    <img\n      [src]=\"employee | employeePhotoUrl:employeePhotos | async\"\n      [style.width]=\"size\"\n      [style.height]=\"size\"\n      [style.border-radius]=\"notRound ? 'default' : '100%'\">\n  }\n}", styles: [""], dependencies: [{ kind: "pipe", type: i1.AsyncPipe, name: "async" }, { kind: "pipe", type: EmployeePhotoUrlPipe, name: "employeePhotoUrl" }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: EmployeePhotoComponent, decorators: [{
            type: Component,
            args: [{ selector: 'employee-photo', template: "@if (employee) {\n  @if ((employee | employeePhotoUrl:employeePhotos | async)) {\n    <img\n      [src]=\"employee | employeePhotoUrl:employeePhotos | async\"\n      [style.width]=\"size\"\n      [style.height]=\"size\"\n      [style.border-radius]=\"notRound ? 'default' : '100%'\">\n  }\n}" }]
        }], propDecorators: { employee: [{
                type: Input
            }], accountName: [{
                type: Input
            }], employeePhotos: [{
                type: Input
            }], size: [{
                type: Input
            }], notRound: [{
                type: Input
            }] } });

class SharePointModule {
    static forRoot(config) {
        return {
            ngModule: SharePointModule,
            providers: [
                SharePointService,
                { provide: ServiceConfig, useValue: config }
            ]
        };
    }
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('SharePointModule is already loaded. Import it in the AppModule only');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointModule, deps: [{ token: SharePointModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.1.2", ngImport: i0, type: SharePointModule, declarations: [EmployeePhotoUrlPipe,
            EmployeePhotoComponent], imports: [CommonModule], exports: [EmployeePhotoUrlPipe,
            EmployeePhotoComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.2", ngImport: i0, type: SharePointModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        EmployeePhotoUrlPipe,
                        EmployeePhotoComponent
                    ],
                    imports: [
                        CommonModule
                    ],
                    exports: [
                        EmployeePhotoUrlPipe,
                        EmployeePhotoComponent
                    ]
                }]
        }], ctorParameters: () => [{ type: SharePointModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }] });

/**
 * Generated bundle index. Do not edit.
 */

export { EmployeePhotoComponent, EmployeePhotoUrlPipe, ServiceConfig, SharePointModule, SharePointService };
//# sourceMappingURL=cleavelandprice-ngx-lib-sharepoint.mjs.map
